import sys, json

if sys.version_info < (3, 6):
    sys.exit("Freqtrade requires Python version >= 3.6")

from pyCryptoTrader.binance import Binance
from pyCryptoTrader.telegram import TelegramBot
from pyCryptoTrader.stagetrading import StageTrading

import logging
from typing import Any, List

logger = logging.getLogger('pyCrptoTrader')

def main():
    if len(sys.argv)<1:
        logger.fatal('missing configuration argument')
    return_code=-1

    try:
        #read config
        with open(sys.argv[1]) as config_file:
            config = json.load(config_file)

        #create instances
        ex=Binance(config['binance'])
        tel=TelegramBot(config,ex)
        cls=dict()
        for key in config['stage'].keys():
            cls[key]=StageTrading(config['stage'][key],ex,tel,key)

        while(True):
            for key in cls.keys():
                cls[key].Run()

    except SystemExit as e:
        return_code = e
    except KeyboardInterrupt:
        with open(sys.argv[1], 'w') as fp:
            json.dump(config, fp)
        logger.info('SIGINT received, aborting ...')
        return_code = 0
    except Exception:
        logger.exception('Fatal exception!')
    finally:
        sys.exit(return_code)

if __name__ == "__main__":
    # execute only if run as a script
    main()
