import ccxt
import logging
logging.basicConfig(format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')

class Binance(ccxt.binance):

    def __init__(self, config): #key, secret):
        super().__init__({'apiKey': config['key'], 'secret': config['secret'],'enableRateLimit': True})
        self.config=config
        self.logger=logging.getLogger('Binance')
        self.logger.setLevel(logging.INFO)

    def Bid(self,symbol):
        try:
            return self.Ticker(symbol)['bid']
        except:
#            self.logger.info('Bid for %s could not be retrieved' %(symbol))
            raise

    def Ticker(self,symbol):
        try:
            return self.fetch_ticker(symbol)
        except Exception as e:
#            self.logger.info('Fetch Ticker for %s failed\n%s' %(symbol,str(e)))
            raise

    def Orders(self,symbol):
        try:
            return self.fetchOrders(symbol)
        except Exception as e:
#            self.logger.info('Fetch Orders for %s failed\n%s' %(symbol,str(e)))
            raise

    def OpenOrders(self, symbol):
        try:
            return self.fetchOpenOrders(symbol)
        except Exception as e:
#            self.logger.info('%s openOrders failed\n %s'  %(symbol,str(e)))
            raise

    def OpenBuys(self, symbol):
        try:
            buys=[]
            orders=self.OpenOrders(symbol)
            for order in orders:
                if order['side']=='buy':
                    buys.append(order)
            return buys
        except:
            raise Exception('%s openBuys Failed' % symbol)

    def OpenSells(self, symbol):
        try:
            sells=[]
            orders=self.OpenOrders(symbol)
            for order in orders:
                if order['side']=='sell':
                    sells.append(order)
            return sells
        except:
            raise Exception('%s openSells Failed' % symbol)

    def MinCost(self,symbol):
        return self.markets[symbol]['limits']['cost']['min']

    def AmountCheck(self,symbol,amount):
        try:
            free=self.Balance(symbol,'quote')
            if amount>free:
                self.logger.info('%s amount requested above available ( %f / %f )'  %(symbol,amount,free))
                raise Exception('%s not enought balance ( %f > %f)' %(symbol,amount,free))
        except:
            return

    def Buy(self, symbol, amount, price):
        try:
            self.AmountCheck(symbol,amount)
            return self.createLimitBuyOrder(symbol, amount, price)
        except Exception as e:
            self.logger.info('%s Buy %f/%f failed : %s' %(symbol,price,amount,str(e)))
            raise Exception(str(e))

    def Sell(self, symbol, amount, price):
        try:
            self.AmountCheck(symbol,amount)
            return  self.createLimitSellOrder(symbol, amount, price)
        except Exception as e:
            self.logger.info('%s Sell Price:%f Amount:%f failed : %s' %(symbol, price, amount, str(e)))
            raise Exception (str(e))

    def Count(self, symbol, type):
        try:
            orders=self.fetchOpenOrders(symbol)
            count=0
            for order in orders:
                if order['side']==type:
                    count=count+1
            return count
        except Exception as e:
   #         self.logger.info('Count %s failed\%s' %(symbol,str(e)))
            raise

    def CancelAll(self, symbol):
        for order in self.OpenOrders(symbol):
            self.Cancel(symbol,order['id'])

    def Cancel(self, symbol, id):
        try:
            return self.cancelOrder(id, symbol)
        except Exception as e:
    #        self.logger.info('Cancel Position %s %s failed\%s' %(symbol,id,str(e)))
            raise

    def Highest(self,symbol,side='buy'):
        try:
            if side=='buy':
                orders=self.OpenBuys(symbol)
            else:
                orders=self.OpenSells(symbol)

            highest=orders[0]
            for order in orders:
                if order['side']!=side:
                    continue
                if order['price']>highest['price']:
                    highest=order
            return highest
        except Exception as e:
     #       self.logger.info('highest %s failed\%s' %(symbol, str(e)))
            raise

    def Lowest(self,symbol,side='buy'):
        try:
            if side=='buy':
                orders=self.OpenBuys(symbol)
            else:
                orders=self.OpenSells(symbol)

            lowest=orders[0]
            for order in orders:
                if order['side']!=side:
                    continue
                if order['price']<lowest['price']:
                    lowest=order
            return lowest
        except Exception as e:
#            self.logger.info('Open Orders %s failed\ %s' %(symbol, str(e)))
            raise

    def Balance(self, symbol, type='quote'): #quote or base
        try:
            currency=self.markets[symbol][type]
            return self.fetchBalance()[currency]['free']
        except Exception as e:
#            self.logger.info('Fetch Balance %s failed\n %s' %(symbol,str(e)))
            raise

    def Profit(self,symbol=''):
        if symbol!='':
            return self.profit[symbol]
        profit=0
        for key in self.profit.keys():
            if self.markets[key]['quote']!=self.config['refCurrency']:
                val=self.Convert(self.markets[key]['quote'],self.config['refCurrency'],self.profit[key])
            else:
                val=self.profit[key]
            profit=profit+val
        return profit

    def Info(self):
        return self.has

    def LastPos(self, symbol, side='buy', offset=0):
        try:
            orders=self.fetchClosedOrders(symbol)
            res=[]
            i=0
            while (len(res)<offset+1):
                if orders[i]['side']==side:
                    res.append(orders[i])
                    i=i+1
            return res[offset]
        except Exception as e:
#            self.logger.info('LastFilled %s failed\n %s' %(symbol,str(e)))
            raise

    def Filled(self, symbol):
        try:
            return self.fetchClosedOrders(symbol)
        except Exception as e:
#            self.logger.info('LastFilled %s failed\n %s' %(symbol,str(e)))
            return []

    def Portfolio(self):
        portfolio=[]
        try:
            results=self.fetchBalance()['total']
            for key in results.keys():
                if results[key]==0:
                    continue
                if key==self.config['refCurrency']:
                    portfolio.append([key,results[key],results[key]])
                    continue
                try:
                    value=self.Convert(key, self.config['refCurrency'], results[key])
                except:
                    continue
                portfolio.append([key,results[key],value])
            return portfolio
        except:
            raise

    def ConvertToReference(self, quote, amount):
        try:
            symbol="%s/%s" %(quote,self.config['refCurrency'])
            bid=self.exchange.Ticker(symbol)['bid']
            return round(amount*bid,self.markets[symbol]['precision']['price'])
        except:
            raise

    def Convert(self, base, quote, amount, mode='quote'):
        try:
            symbol="%s/%s" %(base,quote)
            bid=self.Ticker(symbol)['bid']
            if mode=='quote':
                return round(amount*bid,self.markets[symbol]['precision']['quote'])
            if mode=='base':
                return round(amount/bid,self.markets[symbol]['precision']['base'])
            return 0
        except:
            raise
