from datetime import datetime
from threading import Thread
import logging, math
logging.basicConfig(format='%(asctime)s - %(name)s - %(levelname)s - %(message)s') #,level=logging.NOTSET)

class StageTrading(Thread):

      def __init__(self, config, exchange, telegram, symbol='ETH/USDT'):
            super().__init__()
            self.config=config
            self.symbol=symbol
            self.exchange=exchange
            self.telegram=telegram
            self.start=int(datetime.now().timestamp())
            self.logger=logging.getLogger('StageStrategy')
            self.logger.setLevel(logging.INFO)
            self.status='STOPPED'
            self.lastid=0
            self.count=1
#            self.exchange.profit[symbol]=0
#            self.refAmount=refAmount

      def Run(self):
            if (self.telegram.status=='START') & (self.status=='STOPPED'):
                  self.Start()
                  return
            if (self.telegram.status=='STOP') & (self.status=='RUN'):
                  self.Stop()
                  return
            if self.status=='RUN':
                  self.Update()
                  return
      def PriceUp(self,price):
            return round(price*(1+self.config['step']/100), self.exchange.markets[self.symbol]['precision']['price'])

      def PriceDown(self,price):
            return round(price*(1-self.config['step']/100), self.exchange.markets[self.symbol]['precision']['price'])

      def Start(self): ##start Method
            symbol=self.symbol
            bid=self.exchange.Ticker(self.symbol)['bid']
            self.telegram.Message('%s starting at %f' % (self.symbol,bid))
            self.logger.info('%s starting at %f' % (self.symbol,bid))
#            self.exchange.CancelAll(self.symbol)
            self.status='RUN'

      def Stop(self):
 #           self.exchange.CancelAll(self.symbol)
            self.telegram.Message('%s stopped' % self.symbol)
            self.logger.info('%s Stage Trading Stopped' % self.symbol)
            self.status='STOPPED'

      def Amount(self,price):
            try:
                  minCost=self.exchange.MinCost(self.symbol)
#                  if self.exchange.markets[symbol]['base']!=self.refCurrency:
#                        minCost=self.exchange.Convert(self.exchange.markets[symbol]['base'],self.refCurrency,minCost)
                  factor=self.config['factor']*(1+self.config['profit']/self.config['refAmount'])
                  digits=self.exchange.markets[self.symbol]['precision']['amount']
#                  n=10**-digits
                  val=minCost*factor/price
#                  return round(math.ceil(val/n)*n,digits)
                  return round(val,digits)
#                  return round(minCost*factor/price,self.exchange.markets[self.symbol]['precision']['amount'])
            except Exception as e:
                  self.logger.warning('%s Amount Failed\n %s' %(symbol, str(e)))
                  raise

      def Buy(self, price, amount=0):
            try:
                  amount=self.Amount(price)
                  order=self.exchange.Buy(self.symbol, amount, price)
                  self.logger.info("BUY Amount:%f Price:%f Cost:%f" %(order['amount'], order['price'], order['cost']))
                  return order
            except Exception as e:
#                  self.logger.info("Buy failed : %s" %str(e))
                  return

      def Sell(self, price, amount):
            try:
#                  if amount==0:
#                        amount=self.Amount(price*(1-self.step/100)) #amount from buy position
#                        amount=amount*(1-self.exchange.markets[self.symbol]['taker']/100) #amount corrected with fees
#                        amount=round(amount,self.exchange.markets[self.symbol]['precision']['amount'])
                  order=self.exchange.Sell(self.symbol, amount, price)
                  self.logger.info("SELL Volume:%f Price:%f Cost:%f" %(order['amount'], order['price'], order['cost']))
                  return order
            except Exception as e:
#                  self.logger.info("Sell failed : %s" %str(e))
                  return

      def Profit(self,sell):
            symbol=self.symbol
            filleds=self.exchange.Filled(symbol)
            count=1
            i=2
            while i<len(filleds):
                  filled=filleds[-i]
                  i=i+1
                  if filled['side']=='sell':
                        count=count+1
                  else:
                        count=count-1
                  if count!=0:
                        continue
#                  if filled['amount']==sell['amount']:
                        #if market['quote']!=self.refCurrency:
                        #      buy=self.exchange.Convert(market['quote'],self.refCurrency,filled['cost'])
                        #else:
                  profit=sell['cost']-filled['cost']
                  self.config['factor']=self.config['factor']+profit/self.config['refAmount']
                  self.config['profit']=self.config['profit']+profit
                  self.logger.info('Profit:%f' %profit )
#                  print('Sell:%f Buy:%f' %(sell['cost'],filled['cost']))
                  self.telegram.Message('%s %s Filled Price:%f Amount:%f Cost:%f Profit:%f Facotr:%f' %(symbol, sell['side'], sell['price'], sell['amount'], sell['cost'], profit, self.config['factor')))
#                  self.telegram.Message('Profit:%f' %profit )
                  return
            self.logger.info('No Profit Found !')

      def Update(self): ##update the positions
            symbol=self.symbol
            try:
                  #self.logger.info('Update')
                  filleds=self.exchange.Filled(symbol)
                  filled=filleds[-1]
                  #self.logger.info('%s last:%s fiiled:%s' %(symbol,self.lastid, filled['id']))

                  if self.lastid==0:
                        self.lastid=filled['id']
                        return

                  #self.logger.info('%s last:%s fiiled:%s' %(symbol,self.lastid, filled['id']))

                  if filled['id']==self.lastid:
                        self.UpdateBuys()
                        self.MaxOrders()
                        return

                  self.lastid=filled['id']

                  if filled['side']=='buy':
                        self.FilledBuy(filled)
                  else:
                        self.Profit(filled)
                        self.FilledSell(filled)

            except Exception as e:
                self.logger.info("Update : %s" %str(e))
                #return

      def UpdateBuys(self):
            symbol=self.symbol
            try:
                  bid=self.exchange.Ticker(symbol)['bid']
                  filleds=self.exchange.Filled(symbol)
                  filled=filleds[-1]
#                  self.logger.info('%s update' % symbol)

                  #create buy if none present
                  if self.exchange.Count(symbol,'buy')==0:
                        self.Buy(self.PriceDown(bid))
                        return

                  # update above if price rise up and no sell active
                  if self.exchange.Count(symbol,'sell')==0:
                        highest=self.exchange.Highest(symbol,'buy')
                        p1=self.PriceUp(highest['price'])
                        p2=self.PriceUp(p1)
                        if p2<bid:
                              self.Buy(p1)
                              return

            except Exception as e:
                  self.logger.warning('Updatebuys: %s' %str(e))
                  return

      def FilledBuy(self,filled):
            symbol=self.symbol
            try:
                  #count=self.exchange.Count(symbol,'buy')
                  price=self.PriceDown(filled['price'])
                  if self.exchange.Count(symbol,'buy')==0:
                        self.Buy(price)
                  #else:
                        #update below
                  #      lowest=self.exchange.Lowest(symbol,'buy')
                  #      if price<lowest['price']:
                  #            self.Buy(price)

                  # open a sell
                  #count=self.exchange.Count(symbol,'sell')
                  #if count==0:
                  self.Sell(self.PriceUp(filled['price']),filled['amount'])
                  #else:
                  #      lowest=self.exchange.Lowest(symbol,'sell')['price']
                  #      price=self.PriceUp(filled['price'])
                  #      if price<lowest:
                  #            self.Sell(price,filled['amount'])
            except Exception as e:
                  self.logger.warning('%s FilledBuy: %s' %(symbol,str(e)))

      def FilledSell(self,filled):
        symbol=self.symbol
        try:
            #open buy
            #if self.PriceDown(filled['price'])>self.exchange.Higesht(symbol,'buy')['price']:
            self.Buy(self.PriceDown(filled['price']))

            if self.exchange.Count(symbol,'sell')==0:
                  target=self.PriceUp(filled['price'])
                  bid=self.exchange.Ticker(symbol)['bid']

                  filleds=self.exchange.Filled(symbol)
                  i=2
                  while i<len(filleds):
                        filled=filleds[-i]
                        i=i+1
                        if filled['side']=='sell':
                            continue

                        price=self.PriceUp(filled['price'])
                        if price<target:
                            continue
                        self.Sell(price,filled['amount'])
                        return
        except Exception as e:
            self.logger.warning('%s FilledSell: %s' %(symbol,str(e)))

      def MaxOrders(self):
            symbol=self.symbol
            try:
                  if self.exchange.Count(symbol,'buy')>self.count: #too many position
                        lowest=self.exchange.Lowest(symbol,'buy')
                        self.exchange.Cancel(symbol,lowest['id'])
            except Exception as e:
                  self.logger.warning('MaxOrders Buy failed : %s' %str(e))
            try:
                  if self.exchange.Count(symbol,'sell')>self.count: #too many position
                        highest=self.exchange.Highest(symbol,'sell')
                        self.exchange.Cancel(symbol,highest['id'])
            except Exception as e:
                  self.logger.warning('MaxOrders Sell failed : %s' %str(e))
