#telegram bot
import telegram
from telegram.ext import Updater, CommandHandler, Filters, MessageHandler
from telegram.ext.filters import Filters
from datetime import datetime

import logging
logging.basicConfig(format='%(asctime)s - %(name)s - %(levelname)s - %(message)s', level=logging.INFO)

class TelegramBot(object):

      def __init__(self, config, exchange): #token,id,exchange):
          super().__init__()
          self.config=config
          self.token=self.config['telegram']['token']
          self.id=self.config['telegram']['chat_id']
          self.exchange=exchange
          self.logger=logging.getLogger('TelegramBot')
          self.updater = Updater(token=self.token, use_context=True)
          self.dispatcher = self.updater.dispatcher
          self.dispatcher.add_handler(CommandHandler('start', self.Start, Filters.chat(chat_id=self.id)))
          self.dispatcher.add_handler(CommandHandler('stop', self.Stop, Filters.chat(chat_id=self.id)))
          self.dispatcher.add_handler(CommandHandler(["profitability", "pb"], self.Profitability, Filters.chat(chat_id=self.id)))
          self.dispatcher.add_handler(CommandHandler(["portfolio", "pf"], self.Portfolio, Filters.chat(chat_id=self.id)))
          self.dispatcher.add_handler(CommandHandler('ping', self.Ping, Filters.chat(chat_id=self.id)))
          self.dispatcher.add_handler(CommandHandler('step', self.Step, Filters.chat(chat_id=self.id), pass_args=True))
          self.dispatcher.add_handler(CommandHandler('factor', self.Factor, Filters.chat(chat_id=self.id), pass_args=True))
          self.dispatcher.add_handler(CommandHandler('ref', self.RefAmount, Filters.chat(chat_id=self.id), pass_args=True))
          self.dispatcher.add_handler(CommandHandler('config', self.Config, Filters.chat(chat_id=self.id)))
          self.dispatcher.add_handler(CommandHandler('help', self.Help, Filters.chat(chat_id=self.id)))
          self.dispatcher.add_handler(MessageHandler(Filters.command, self.Unknown, Filters.chat(chat_id=self.id)))
          self.updater.start_polling()
          self.updater.bot.send_message(chat_id=self.id, text="Hello, type /help to know the commands.")
          self.status='STOPPED'
          self.start=datetime.now()

      def __del__(self):
            self.updater.stop_polling()

      def Start(self, update, context):
            self.status='START'
            #context.bot.send_message(chat_id=update.effective_chat.id, text="Started !")

      def Stop(self, update, context):
            self.status='STOP'
            #context.bot.send_message(chat_id=update.effective_chat.id, text="Stopped !")

      def Profitability(self, update, context):
            try:
                  for key in self.config['stage'].keys():
                        market=self.exchange.markets[key]
                        value=self.config['stage'][key]['profit']
                        text='%s %.*f %s' %(key, market['precision']['price'], value, market['quote'])
                        context.bot.send_message(chat_id=update.effective_chat.id, text=text)
                        self.logger.info(text)
                        #            text="Total: %f %s" %(total,,self.exchange.refCurrency)
#                        context.bot.send_message(chat_id=update.effective_chat.id, text=text)
            except Exception as e:
                  self.logger.warning('Profitability Failed:%s' % str(e))


      def Portfolio(self, update, context):
            try:
                  total=0
                  results=self.exchange.Portfolio()
                  total=0
                  for result in results:
                        text="%s %f %f %s" %(result[0], result[1], result[2],self.exchange.config['refCurrency'])
                        total=total+result[2]
                        context.bot.send_message(chat_id=update.effective_chat.id, text=text)
                  text="Total: %f %s" %(total,self.exchange.config['refCurrency'])
                  context.bot.send_message(chat_id=update.effective_chat.id, text=text)
            except Exception as e:
                  self.logger.warning('Portfolio Failed:%s' % str(e))

      def Ping(self, update, context):
            text="Stage Trading alive since %s" %(self.start.strftime("%Y-%m-%d %H:%M:%S"))
            context.bot.send_message(chat_id=update.effective_chat.id, text=text)

      def OpenOrders(self, update, context):
            pass
#            results=self.exchange.OpenOrders()
#            for result in results:
#                  context.bot.send_message(chat_id=update.effective_chat.id, text="Order %s Amount:%f Price:%f Cost%s" %(result['type'], result['amount'], result['price'], result['cost'])

      def Unknown(self, update, context):
            context.bot.send_message(chat_id=update.effective_chat.id, text="`Unfortunately, I don't know the command: %s`" %(update.effective_message.text))

      def Step(self, update, context):
            args=context.args
            try:
                  if len(args)<2:
                        raise Exception('Missing arguments')
                  self.config['stage'][args[0]]['step']=float(args[1])
            except Exception as e:
                  self.logger.warnine('Step Failed:%s' % str(e))

      def Factor(self, update, context):
            args=context.args
            try:
                  if len(args)<2:
                        raise Exception('Missing arguments')
                  self.config['stage'][args[0]]['factor']=float(args[1])
            except Exception as e:
                  self.logger.warnine('Factor Failed:%s' % str(e))

      def RefAmount(self, update, context):
            args=context.args
            try:
                  if len(args)<2:
                        raise Exception('Missing arguments')
                  self.config['stage'][args[0]]['refAmount']=float(args[1])
            except Exception as e:
                  self.logger.warnine('RefAmount Failed:%s' % str(e))

      def Config(self, update, context):
            for key in self.config['stage'].keys():
                  market=self.exchange.markets[key]
                  conf=self.config['stage'][key]
                  text="%s\n" % key
                  text="%sStep:%.1f\n" %(text,conf['step'])
                  text="%sFactor:%f\n" %(text, conf['factor'])
                  text="%sRefAmount:%.*f\n" %(text,market['precision']['amount'],conf['refAmount'])
                  text="%sProfit:%.*f\n" %(text, market['precision']['price'], conf['profit'])
                  context.bot.send_message(chat_id=update.effective_chat.id, text=text, parse_mode=telegram.ParseMode.MARKDOWN)


      def Help(self, update, context):
            message = "* - My Bot Commands - *\n"
            message += "/start: `Displays my startup message.`\n"
            message += "/ping: `Shows for how long I'm working.`\n"
#            message += "/portfolio or /pf: `Displays my current portfolio.`\n"
#            message += "/open\_orders or /oo: `Displays my current open orders.`\n"
#            message += "/trades\_history or /th: `Displays my trades history since I started.`\n"
            message += "/profitability or /pb: `Displays the profitability I made since I started.`\n"
            message += "/stop: `Stops me.`\n"
            message += "/help: `Displays this help.`"
            context.bot.send_message(chat_id=update.effective_chat.id, text=message, parse_mode=telegram.ParseMode.MARKDOWN)

      def Message(self, text):
            self.updater.bot.send_message(chat_id=self.id, text=text)
